﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Windows.Forms;

namespace Paketbote
{
    /// <summary>
    /// class for custom buttons
    /// </summary>
    class custom_Button : Button
    {
        private static Font _normalFont = new Font("Arial", 20F, FontStyle.Bold, GraphicsUnit.Point, ((byte)(0)));

        private static Color _back = Color.White;
        private static Color _border = Color.FromArgb(64, 64, 64);
        private static Color _activeBorder = Color.FromArgb(255, 152, 0);
        private static Color _fore = Color.FromArgb(64, 64, 64);

        private static Padding _margin = new Padding(0, 0, 0, 0);
        private static Padding _padding = new Padding(3, 3, 3, 3);

        private bool _active = false;
        private bool _hover = false;

        private SolidBrush _brushText,  _brushBack, _brushTextActive;
        private Pen _penBorder, _penActiveBorder;
        private Rectangle _rectBorder;
        private StringFormat stringFormat = new StringFormat();

        public override Cursor Cursor { get; set; } = Cursors.Hand;
        public int _borderThickness { get; set; } = 4;

        public custom_Button(string _text,Point _location, Size _size)
            :base()
        {
            Text = _text;
            Location = _location;
            Size = _size;

            Font = _normalFont;
            BackColor = _back;
            ForeColor = _fore;
            FlatAppearance.BorderColor = _border;
            FlatStyle = FlatStyle.Flat;
            Margin = _margin;
            Padding = _padding;
            
            _penBorder = new Pen(_border, _borderThickness);
            _penBorder.Alignment = System.Drawing.Drawing2D.PenAlignment.Inset;
            _penActiveBorder = new Pen(_activeBorder, _borderThickness);
            _penActiveBorder.Alignment = System.Drawing.Drawing2D.PenAlignment.Inset;
            _brushText = new SolidBrush(_fore);
            _brushTextActive = new SolidBrush(_activeBorder);
            _brushBack = new SolidBrush(_back);

            stringFormat.Alignment = StringAlignment.Center;
            stringFormat.LineAlignment = StringAlignment.Center;

            _rectBorder = new Rectangle(0, 0, Width, Height);

            Paint += custom_Button_Paint;
        }

        private void custom_Button_Paint(object sender, PaintEventArgs e)
        {
            if (_hover)
            {
                e.Graphics.FillRectangle(_brushBack, _rectBorder);
                e.Graphics.DrawRectangle(_penActiveBorder, _rectBorder);
            }
            else
            {
                e.Graphics.FillRectangle(_brushBack, _rectBorder);
                e.Graphics.DrawRectangle(_penBorder, _rectBorder);
            }
            if (_active)
            {
                e.Graphics.DrawString(Text, Font, _brushTextActive, _rectBorder, stringFormat);
            }
            else
            {
                e.Graphics.DrawString(Text, Font, _brushText, _rectBorder, stringFormat);
            }         
        }

        protected override void OnMouseEnter(EventArgs e)
        {
            base.OnMouseEnter(e);
            _hover = true;
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            _hover = false;
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            _active = true;
            Invalidate();
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);
            _active = false;
            Invalidate();

        }

        public void SetStateActive()
        {
            _active = true;
        }

        public void SetStateNormal()
        {
            _active = false;
        }
    }
}
